/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
//@@@@@@@@@@@@@@@@
function XPI_Installer		//<c><a>public<d>XPI installer class
(
_descriptor		//<p>string<d>Application descriptor
){
//================
this.XPI_Installer = function		//<m>void<a>private<d>class constructor
(
descriptor		//<p>string<d>Application descriptor
){
this.className	= 'XPI_Installer';
this.descriptor	= descriptor;
this.setIn();
}//</m>XPI_Installer



//================
this.setIn = function		//<m>int<a>private<d>Instance i12n
(){
if(Install.initInstall(this.descriptor.label, this.descriptor.pack, this.descriptor.version) != this.K.error.NONE) return this.K.error.NO_SUCCESS;
this.l10nSet = Install.loadResources(this.descriptor.l10n);
Install.logComment('this.l10nSet is null:\n' + !this.l10nSet + '\n');
if(!this.l10nSet) return this.setAbort(this.K.error.INSTALL_BUNDLE);
if(this.testSpace(this.descriptor.target) != this.K.error.NONE) return this.setAbort(this.K.error.INSUFFICIENT_DISK_SPACE);
this.setOutLog(this.K.l10n.INSTALL_START, new Date(), this.descriptor.label, this.descriptor.pack, this.descriptor.version);
return this.K.error.NONE;
}//</m>setIn



//================
this.setUp = function		//<m>int<a>public<d>Set up installation parameters
(
monitor			//<p>Object<d>Installation monitor singleton
){
if(monitor.setUp(this) == this.K.error.NONE) return this.setInstall();
			///***** DEBUG ****/if(monitor.setUp(this) == this.K.error.NONE) return;
}//</m>setUp



//================
this.setInstall = function		//<m>int<a>private<d>Proceed to installation
(){
var
/*int*/code = Install.performInstall();
this.setOutLog(this.K.l10n.INSTALL_END, code);
return code;
}//</m>setInstall



//================
this.setAbort = function		//<m>int<a>public<d>Abort installation process
(
code,		//<p>int<d>Error code
logging		//<p>boolean<d>Log method flag: Install::logComment || XPI_Installer::setOutLog
){
if(logging) Install.logComment(code);
else this.setOutLog(this.K.l10n.INSTALL_ERROR, code);
Install.cancelInstall(code);
return this.K.error.NO_SUCCESS;
}//</m>setAbort



//================
this.setOutLog = function		//<m>void<a>private<d>Log to the installation log file
(
property		//<p>string<d>Property for l10n
//propertySet	//<p>string[]<d>Run time property set to insert in property text
){
var
content = new String(this.l10nSet[property]);
for(var i=1,m=arguments.length;i<m;i++) content += ' -- ' + arguments[i];
Install.logComment(content);
return;

/*############################################# DEBUG ##############################################*/
/*string*/log = '',
/*string[]*/logSet = content.split(/%s/g);
if(logSet) for(var i=1,m=logSet.length+1;i<m;i++) log += logSet[i-1] + ((arguments[i] != undefined) ? arguments[i] : '');
else log = content;
Install.logComment(log);
/*############################################# DEBUG ##############################################*/
}//</m>setOutLog<d>&
/*
Modified from sprintf function
*/



//================
this.setAddSource = function		//<m>boolean(int)<a>protected<d>Add a file/dir from source archive to the target
(
type,			//<p>boolean<d>Source flag: file || dir														DIR_SOURCE
source,			//<p>string<d>File/dir source path															mozlib
destination,	//<p>string<d>File/dir destination path														Chrome
target,			//<p>string<d>Sub directory or file rename --optionnal
replacing		//<p>boolean<d>Replacing old version flag: force || skip --optionnal
){
target		= target || '';			//optionnal arg
replacing	= replacing || true;	//optionnal arg
var
/*string*/code,
/*string*/method = (type == this.K.DIR_SOURCE) ? 'addDirectory' : 'addFile';
if(arguments.length == 2) code = Install[method](source);
else{
	var
	/*XPI_FileHandler*/destFile =  new XPI_FileHandler(destination);
	code = Install[method](this.descriptor.register, this.descriptor.version, source, destFile.get_handle(), target, replacing);
	}
if(code != this.K.error.NONE) return this.setAbort(this.K.error.ADD_SOURCE);
this.setOutLog(this.K.l10n.INSTALL_SOURCE, source, destination, target);
return this.K.error.NONE;
}//</m>setAddSource



//================
this.setRegister = function		//<m>boolean(int)<a>protected<d>Regsiter parckage content with chrome registry
(
mode,			//<p>int<d>Registring mode flag: delayed, etc.
path,			//<p>string<d>RDF manifest (contents.rdf) directory path
jarPath			//<p>string<d>RDF manifest (contents.rdf) jar directory path
){
try{
	var
	/*XPI_FileHandler*/packHandle =  new XPI_FileHandler(this.descriptor.target, this.descriptor.pack),
	/*Object*/regHandle = packHandle.setAddPath(path),
	/*int*/code = jarPath ? Install.registerChrome(mode, regHandle, jarPath) : Install.registerChrome(mode, regHandle);
	if(code != this.K.error.NONE) return this.setAbort(this.K.error.CHROME_REGISTRY_ERROR);
	this.setOutLog(this.K.l10n.INSTALL_REGISTER, path);
	return this.K.error.NONE;
	}
catch(e){
	return this.K.error.NO_SUCCESS;
	}
}//</m>setRegister



//================
this.testSpace = function		//<m>boolean(int)<a>private<d>Check wether available disk space is OK to perform install
(
target		//<p>string<d>Target path
){
try{
	var
	/*XPI_FileHandler*/volume = new XPI_FileHandler(target),
	/*int*/available = volume.getSpace() / 1024;
	if(available > this.descriptor.size) return this.K.error.NONE;
	this.setOutLog(this.K.l10n.INSTALL_SPACE, this.descriptor.target, this.descriptor.size, available);
	return this.K.error.NO_SUCCESS;
	}
catch(e){
	return this.K.error.NO_SUCCESS;
	}

}//</m>testSpace



this.K =
	{
	FILE_SOURCE:				0,
	DIR_SOURCE:					1,

	l10n:
		{
		INSTALL_BUNDLE:			'INSTALLATION ERROR: Localization file not found.',
		INSTALL_ERROR:			'xpi.install.error',		//l10n.properties entries
		INSTALL_START:			'xpi.install.start',
		INSTALL_END:			'xpi.install.end',
		INSTALL_TARGET:			'xpi.install.target',
		INSTALL_SPACE:			'xpi.install.space',
		INSTALL_SOURCE:			'xpi.install.source',
		INSTALL_REGISTER:		'xpi.install.register',
		},

	error:
		{
		//NON FATAL
		NONE:					0,
		NO_SUCCESS:				1,

		//FATAL
		ADD_SOURCE:				-2,
		INSUFFICIENT_DISK_SPACE:-235,		//XPI Install constants
		INSTALL_BUNDLE:			-238,
		CHROME_REGISTRY_ERROR:	-239,
		FATAL:				0xFF		//end of error codes
		}
	};



this.XPI_Installer(_descriptor);
}//</c>XPI_Installer



//@@@@@@@@@@@@@@@@
function XPI_FileHandler		//<c><a>protected<d>File handler class
(
_path,		//<p>string<d>File/dir path
_subDir		//<p>string<d>Sub idrectory path
){
//================
this.XPI_FileHandler = function		//<m>void<a>protected<d>class constructor
(
path,		//<p>string<d>File/dir path
subDir		//<p>string<d>Sub idrectory path --optionnal
){
this.className	= 'XPI_FileHandler';
this.path		= path;
this.subDir		= subDir || '';
this.setIn();
}//</m>XPI_FileHandler



//================
this.setIn = function		//<m>void<a>private<d>Instance i12n
(){
try{
	this.handle = Install.getFolder(this.path, this.subDir);
	}
catch(e){
	this.handle = null;
	}
}//</m>setIn



//================
this.setLeaf = function		//<m>int<a>protected<d>Set up a new name for the handled file/dir
(
leaf			//<p>string<d>New name
){
try{
	var
	/*string*/method = this.testFile() ? 'rename' : 'dirRename';
	File[method](this.handle, leaf);
	return this.K.error.NONE;
	}
catch(e){
	return this.K.error.NO_SUCCESS;
	}
}//</m>setLeaf



//================
this.setPath = function		//<m>int<a>protected<d>Set up a new path for the handled file/dir
(
path,			//<p>string<d>Target directory path or null to remove the file
preserving		//<p>boolean<d>Actual file status flag: copy || move --optional
){
if(!path) return this.setUpPath(this.testFile() ? 'remove' : 'dirRemove', this.K.DIR_SINKING);
var
/*Object*/target = Install.getFolder(path);
if(!target) return this.K.error.NO_SUCCESS;
return preserving ? this.setUpPath('copy', target) : this.setUpPath('move', target);
}//</m>setPath<d>&
/*
Set up the path to copy, move or remove the handled file/dir
*/



//================
this.setAddPath = function		//<m>Object<a>protected<d>Set up a sub path from the handled dir
(
path			//<p>string<d>Sub directory path
){
return this.testHandle() ? Install.getFolder(this.handle, path) : null;
}//</m>setAddPath



//================
this.setUpPath = function		//<m>int<a>private<d>Move/Copy the handled file/dir
(
method,		//<p>string<d>Set up method
target		//<p>XPCOM_nsIFile<d>Target directory handle
){
try{
	File[method](this.handle, target);
	return this.K.error.NONE;
	}
catch(e){
	return this.K.error.NO_SUCCESS;
	}
}//</m>setUpPath



//================
this.setOut = function		//<m>int<a>public<d>Create the directory of the handle dir
(){
try{
	if(!this.testHandle()) return this.K.error.NO_SUCCESS;
	File.dirCreate(this.handle);
	return this.K.error.NONE;
	}
catch(e){
	return this.K.error.NO_SUCCESS;
	}
}//</m>setOut



//================
this.get_handle = function		//<m>Object<a>protected<d>Get the handle member
(){
return this.handle;
}//</m>get_handle



//================
this.getSpace = function		//<m>int<a>protected<d>Get the space available for the directory
(){
try{
	return this.testExists() ? File.diskSpaceAvailable(this.handle) : 0;
	}
catch(e){
	return 0;
	}
}//</m>getSpace



//================
this.getParent = function		//<m>Object<a>protected<d>Get the parent directory for the handled file/dir
(){
try{
	return this.testHandle() ? File.dirGetParent(this.handle) : null;
	}
catch(e){
	return null;
	}
}//</m>getParent



//================
this.getDate = function		//<m>string<a>protected<d>Get date modified in UNIX epoch format
(){
try{
	return this.testExists() ? File.modDate(this.handle) : '';
	}
catch(e){
	return '';
	}
}//</m>getDate



//================
this.testHandle = function		//<m>boolean<a>protected<d>Check wether handle is valid
(){
return this.handle != null;
}//</m>testHandle



//================
this.testExists = function		//<m>boolean<a>protected<d>Check wether handle references an existing file/dir
(){
return this.testHandle() ? File.exists(this.handle) : false;
}//</m>testExists



//================
this.testFile = function		//<m>boolean<a>protected<d>Check wether handle references a file
(){
return this.testHandle() ? File.isFile(this.handle) : false;
}//</m>testFile



//================
this.testDirectory = function		//<m>boolean<a>protected<d>Check wether handle references a directory
(){
return this.testHandle() ? File.isDirectory(this.handle) : false;
}//</m>testDirectory



this.K =
	{
	DIR_SINKING:			true,

	error:
		{
		//NON FATAL
		NONE:				0,
		NO_SUCCESS:			1,

		//FATAL
		FATAL:				0xFF		//end of error codes
		}
	};



this.XPI_FileHandler(_path, _subDir);
}//</c>XPI_FileHandler




//################
function main		//<f>void<a>public<d>Specific installation
(){
const
appDescriptor =
	{
	label:		'Mozilla library',
	register:	'/mozdev/mozlib/library',
	pack:		'mozlib',
	version:	'1.5',
	target:		'Chrome',
	size:		360000,
	l10n:		'l10n.properties'
	};
var
installMonitor =
	{
	setUp: function		//Installer callback method
	(
	installer	//<p>XPI_Installer<d>Installer
	){
	if ((installer.setAddSource(installer.K.DIR_SOURCE, appDescriptor.pack, appDescriptor.target, appDescriptor.pack)) ||
		(installer.setRegister(Install.CONTENT | Install.DELAYED_CHROME, 'content')) ||
		(installer.setRegister(Install.LOCALE | Install.DELAYED_CHROME, 'locale/en-US')) ||
		(installer.setRegister(Install.SKIN | Install.DELAYED_CHROME, 'skin'))) return this.error.NO_SUCCESS;
	return this.error.NONE;
	},
	error:
		{
		NONE:				0,
		NO_SUCCESS:			1
		}
	};
var
XPIInstaller = new XPI_Installer(appDescriptor);
XPIInstaller.setUp(installMonitor);
}//</f>main
main();

