/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
//@@@@@@@@@@@@@@@@
function DATETIME_Formatter		//<c><a>public<d>Date, time and duration formatting class
(){
//================
this.DATETIME_Formatter = function		//<m>void<a>private<d>class constructor
(){
this.className	= 'DATETIME_Formatter';
}//</m>DATETIME_Formatter


//================
this.getISODateTime =
this.getDateTime = function		//<m>string<a>public<d>Format a date
(
dateTime,		//<p>Object<d>Date
spacing			//<p>boolean=false<d>ISO-8601 formatting flag: blank time separator || true ISO format --optional
){
try{
	var
	/*int*/millisec = dateTime.getMilliseconds();
	if(millisec < 100) millisec = '0' + millisec;
	return sprintf(this.K.ISO_DATETIME_FORMAT,
		dateTime.getFullYear(),
		this.getLeadingZero(dateTime.getMonth()+1),
		this.getLeadingZero(dateTime.getDate()),
		spacing ? this.K.ISO_TIME_SPACER : this.K.ISO_TIME_DELIMITER,
		this.getLeadingZero(dateTime.getHours()),
		this.getLeadingZero(dateTime.getMinutes()),
		this.getLeadingZero(dateTime.getSeconds()),
		this.getTrailingMilliseconds(millisec));
	}
catch(e){
	return this.K.INVALID_ISO_DATE;
	}
}//</m>getDateTime



//================
this.getISO2UTC = function		//<m>int<a>public<d>Convert an ISO format date to a UTC date
(
isoDate		//<p>string<d>Date in ISO format
){
try{
	var
	/*string*/isoSet = isoDate.match(this.K.ISO_DATETIME_PATTERN);
	return Date.UTC(isoSet[1], isoSet[2], isoSet[3], isoSet[4], isoSet[5], isoSet[6], isoSet[8] || 0);
	}
catch(e){
	return this.K.INVALID_UTC_DATE;
	}
}//</m>getDate


//================
this.getUTC2ISO = function		//<m>int<a>public<d>Convert an UTC date to an ISO format date
(
utcdate,		//<p>string<d>Date in UTC format (miliseconds since 1970-01-01)
spacing			//<p>boolean=false<d>ISO-8601 formatting flag: blank time separator || true ISO format --optional
){
try{
	var
	utcDate = new Date(utcdate),
	/*int*/millisec = utcDate.getUTCMilliseconds();
	if(millisec < 100) millisec = '0' + millisec;
	return sprintf(this.K.ISO_DATETIME_FORMAT,
		utcDate.getUTCFullYear(),
		this.getLeadingZero(utcDate.getUTCMonth()),
		this.getLeadingZero(utcDate.getUTCDate()),
		spacing ? this.K.ISO_TIME_SPACER : this.K.ISO_TIME_DELIMITER,
		this.getLeadingZero(utcDate.getUTCHours()),
		this.getLeadingZero(utcDate.getUTCMinutes()),
		this.getLeadingZero(utcDate.getUTCSeconds()),
		this.getTrailingMilliseconds(utcDate.getUTCMilliseconds()));
	}
catch(e){
	return this.K.INVALID_ISO_DATE;
	}
}//</m>getDate


//================
this.getLeadingZero = function		//<m>string<a>private<d>Add a leading zero to format a datetime section
(
value		//<p>string<d>Value
){
var
/*string*/lead = (value < 10) ? '0' : '';
return lead + value;
}//</m>getLeadingZero



//================
this.getTrailingMilliseconds = function		//<m>string<a>private<d>Get milliseconds format
(
millisec		//<p>string<d>Milliseconds value
){
return (millisec != '00') ? '.' + this.getLeadingZero(millisec) : '';
}//</m>getTrailingMilliseconds



//================
this.testISO = function		//<m>boolean<a>public<d>Check whether a dateTime string conforms to ISO-8601 format
(
dateTime		//<p>string<d>dateTime to check
){
try{
	return dateTime.toString().match(this.K.ISO_DATETIME_PATTERN);
	}
catch(e){
	return false;
	}
}//</m>testISO



//================
this.testISOMilliseconds = function		//<m>boolean<a>public<d>Check whether a dateTime string conforms to ISO-8601 format
(
dateTime		//<p>string<d>dateTime to check
){
try{
	return dateTime.toString().match(this.K.ISO_MILLI_PATTERN);
	}
catch(e){
	return false;
	}
}//</m>testISOMilliseconds



this.K =
	{
	ISO_DATETIME_PATTERN:	/(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(\.(\d{1,3}))?/,
	ISO_MILLI_PATTERN:		/\.\d{1,3}$/,
	ISO_DATETIME_FORMAT:	'%s-%s-%s%s%s:%s:%s%s',
	ISO_TIME_DELIMITER:		'T',
	ISO_TIME_SPACER:		' ',
	INVALID_ISO_DATE:		'000-00-00T00:00:00',
	INVALID_UTC_DATE:		0
	};



this.DATETIME_Formatter();
}//</c>DATETIME_Formatter

