/*
 * The contents of this file are
 * published under the Mozilla Public License 1.1
 */
//@@@@@@@@@@@@@@@@
function IO_file_Locator		//<c><a>protected<d>File locator class
(){
//================
this.IO_file_Locator = function		//<m>void<a>private<d>class constructor
(){
this.className		= 'IO_file_Locator';
this.setIn();
}//</m>IO_file_Locator



//================
this.setIn = function		//<m>void<a>private<d>Instance i12n
(){
this.errorNotifier	= new ERROR_Notifier(this.className);
this.setXPCOM();
}//</m>setIn



//================
this.setXPCOM = function		//<m>void<a>private<d>Set up XPCOM interfaces
(){
try{
	this.dirService = XPConnector.getConstructor('nsIProperties');
	return this.K.ERROR.NONE;
	}
catch(e){
	return this.errorNotifier.setOut(e, this.K.ERROR.SERVICE_ACCESS);
	}
}//</m>setXPCOM



//================
this.setPath = function		//<m>void<a>public<d>Set up a new path for the handled file
(
path,			//<p>string<d>Target directory path or null to remove the file
preserving,		//<p>boolean<d>Actual file status flag: copy || move --optional
leaf			//<p>string<d>New leafName for handled file --optional
){
if(!path) return this.setUpPath('remove', this.K.DIR_SINKING, '', 'NO_SUCCESS');
var
/*XPCOM_nsIFile*/target = this.getPathHandle(path);
return (typeof(target) == 'number') ? target :						//error.TARGET_URL_ENTITY
	preserving ? this.setUpPath('copyTo', target, leaf, 'NO_SUCCESS') :
		this.setUpPath('moveTo', target, leaf, 'NO_SUCCESS');
}//</m>setPath<d>&
/*
Set up the path to copy, move or remove the handled file/dir
*/



//================
this.setUpPath = function		//<m>int<a>private<d>Remove/Move/Copy the handled file/dir
(
method,		//<p>string<d>Set up method
target,		//<p>XPCOM_nsIFile<d>Target directory handle
leaf,		//<p>string<d>New leafName for handled file --optional
code		//<p>int<d>Processing code
){
try{
	this.handle[method](target, leaf || '');
	return this.K.ERROR.NONE;
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.ERROR[code]);
	return this.K.ERROR.NO_SUCCESS;
	}
}//</m>setUpPath



//================
this.getPathHandle = function		//<m>XPCOM_nsIFile<a>private<d>Get directory target path handle
(
path			//<p>string<d>Target directory path
){
var
/*XPCOM_nsIFile*/handle = this.handle.clone(),						//preserve this handle (source)
/*IO_file_dirInstantiator*/target = new IO_file_dirInstantiator();	//FIXME: target instantiation overwrite source handle. Why?
target.set_handle(path);
this.handle = handle;												//restore this handle (source)
if(target.testProcessing('testDirectory()') != target.K.error.NONE) return target.K.error.TARGET_URL_ENTITY;
return target.get_handle();
}//</m>getPathHandle



//================
this.getUrl = function		//<m>string<a>public<d>Get an URL about the handled file/dir
(
fragment		//<p>string<d>Fragment to get: URL_URL (default) || URL_PATH || URL_LEAF || URL_NAME || URL_EXTENSION
){
const
URL_URL			= this.K.URL_URL,
URL_PATH		= this.K.URL_PATH,
URL_LEAF		= this.K.URL_LEAF,
URL_NAME		= this.K.URL_NAME,
URL_EXTENSION	= this.K.URL_EXTENSION;
try{
	var
	/*string*/url;
	switch(fragment){
		case URL_PATH:
			url = this.getPath(this.handle.leafName);
			break;
		case URL_LEAF:
			url = this.handle.leafName;
			break;
		case URL_NAME:
			url = this.getName(this.handle.leafName);
			break;
		case URL_EXTENSION:
			url = this.getExtension(this.handle.leafName);
			break;
		default:	//URL_URL:
			url = this.handle.path;
			break;
		}	//---switch
	return url;
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.ERROR.URL_READ);
	return null;
	}
}//</m>getUrl



//================
this.getPath = function		//<m>string<a>private<d>Extract the path from a file/dir leaf
(
leaf		//<p>string<d>Leaf to extract name
){
return this.handle.path.substr(0, this.handle.path.indexOf(leaf));
}//</m>getPath



//================
this.getMozHandle = function		//<m>string<a>public<d>Get the handle of a Mozilla application directory
(
key		//<p>string<d>Application key
){
try{
	var
	/*Object*/mozDir = new this.dirService(key);
	return mozDir.get(key, XPConnector.getInterface('nsIFile'));
	}
catch(e){
	this.errorNotifier.setOut(e, this.K.ERROR.MOZ_ACCESS);
	return '';	
	}
}//</m>getMozHandle



//================
this.getMozPath = function		//<m>string<a>public<d>Get the path of a Mozilla application directory
(
key		//<p>string<d>Application key
){
return this.getMozHandle(key).path;
}//</m>getMozPath



//================
this.getName = function		//<m>string<a>private<d>Extract the name from a file leaf
(
leaf		//<p>string<d>Leaf to extract name
){
var
/*int*/atDot = leaf.lastIndexOf('.');
return (atDot > -1) ? leaf.substr(0, atDot) : leaf;
}//</m>getName



//================
this.getExtension = function		//<m>string<a>private<d>Extract the extension from a file leaf
(
leaf		//<p>string<d>Leaf to extract extension
){
var
/*int*/atDot = leaf.lastIndexOf('.');
return (atDot > -1) ? leaf.substr(atDot + 1) : '';
}//</m>getExtension




//================
this.getLinkTarget = function		//<m>string<a>public<d>Get the path pointed to by the handled file/dir symbolic link
(){
try{
	return this.handle.target;
	}
catch(e){
	return '';		//not a symbolic link
	}
}//</m>getLinkTarget



//================
this.getParent = function		//<m>string<a>public<d>Get the handled file/dir parent handle
(){
return this.handle.parent;
}//</m>getParent



//================	NOT IMPLEMENTED
this.getPathSet = function		//<m>string[]<a>public<d>Get fragments of path from the handled file/dir to that of the arg
(
path				//<p>string<d>Destination path (ascendance || descendance)
){

}//</m>getPathSet



//================	NOT IMPLEMENTED
this.testAncestor = function	//<m>boolean<a>public<d>Check whether the path of the handled file/dir is an ancestor of the argument one
(
url				//<p>string<d>Potential descendant URL
){

}//</m>testAncestor



//================	NOT IMPLEMENTED
this.testDescendant = function		//<m>boolean<a>public<d>Check whether the path of the handled file/dir is a descendant of the argument one
(
url				//<p>string<d>Potential ancestor URL
){

}//</m>testDescendant



this.K =
	{
	URL_URL:			0,		//complete URL
	URL_PATH:			1,		//leaf stripped
	URL_LEAF:			2,		//path stripped
	URL_NAME:			3,		//leaf extension stripped
	URL_EXTENSION:		4,		//leaf name stripped
	
	DIR_SINKING:		true,

	ERROR:
		{
		//NON FATAL
		NONE:					0,
		NO_SUCCESS:				1,

		//FATAL
		SERVICE_ACCESS:			-1,
		URL_READ:				-2,
		URL_REMOVE:				-3,
		URL_MOVE:				-4,
		URL_COPY:				-3,
		MOZ_ACCESS:				-5,		//Mozilla directory access

		FATAL:					0xFF		//end of error codes
		}
	};



this.IO_file_Locator();
}//</c>IO_file_Locator<d>&
/*
This component class manages handled file/dir path:
	input to read actual path,
	output to change actual location (move/remove) or create a copy
*/




